/*
 * dx-broadcast.c - A AX.25 broadcasting DX cluster client - version 1.0
 *
 * Copyright (C) 2011 Marius Petrescu, YO2LOJ, <marius@yo2loj.ro>.
 *
 * Compile with: gcc -Wall -O6 -lax25 -o dx-broadcast dx-broadcast.c
 *
 *
 * Usage: dx-broadcast -p <port> [-c <my_call>] [-d <dest_call>] [-u <user>] <host> [<service>]
 *
 * Options:
 *          -p <port>             ax.25 port to use
 *
 *          -c <my_call>          Use <my_call> as outgoing call
 *                                Default port call will be used if
 *                                not defined
 *
 *          -d <dest_call>        Use <dest_call> as destination
 *                                DXC-0 will be used if not defined
 *
 *          -u <user>             Use <user> to login into cluster
 *
 *          <host>                Host name of the DX cluster
 *
 *          <service>             Service port of the DX cluster
 *                                Defaults to 8873
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Version History
 * ---------------
 * 	1.0	2.Dec.2011	First release
 * 	1.1	11.Dec.2011	Changed to blocking mode
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <netax25/ax25.h>
#include <netax25/axlib.h>
#include <netax25/axconfig.h>
#include <netax25/daemon.h>

#define	BUFFERSIZE	4096


static char *usage_string = "Usage: dx-broadcast -p <port> [-c <my_call>] [-d <dest_call>] -u <user> <host> [<service>]\n";



#define PERROR(s)	fprintf(stderr, "%s: %s\n", (s), strerror(errno))



int main(int argc, char **argv)
{
	char inbuf[BUFFERSIZE];

	int len, dlen;
	
	int login = 0;

	char *port = NULL;
	char *mycall = NULL;
	char *dstcall = "DXC";
	char *user = NULL;
	int axfd, ipfd;
	
	struct sockaddr_in sin;
        struct hostent *hp;
	struct servent *sp;
	
	char addr[20];
	char *portcall;
	struct full_sockaddr_ax25 src, dest;
	

	while ((len = getopt(argc, argv, "p:c:d:u:")) != -1) {
		switch (len) {
		case 'p':
			port = optarg;
			break;
		case 'c':
			mycall = optarg;
			break;
		case 'd':
			 dstcall = optarg;
			break;
		case 'u':
			 user = optarg;
			break;
		case ':':
		case '?':
			fprintf(stderr, "%s", usage_string);
			return 1;
		}
	}

	if (optind == argc) {
		fprintf(stderr, "%s", usage_string);
		return 1;
	}

	if ((ipfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		PERROR("dx-broadcast: socket");
		return 1;
	}

	if ((hp = gethostbyname(argv[optind])) == NULL) {
		fprintf(stderr, "Unknown host %s\n", argv[optind]);
		close(ipfd);
		return 1;
	}
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;

	sin.sin_port = htons(8873);
	
	if (optind == argc - 2) {
		sp = getservbyname(argv[optind + 1], "tcp");
		if (sp != NULL)
			sin.sin_port = sp->s_port;
		else
			sin.sin_port = htons(atoi(argv[optind + 1]));

		if (ntohs(sin.sin_port) == 0) {
			fprintf(stderr, "Unknown service %s\n", argv[optind + 1]);
			return 1;
		}
	}

	if (port == NULL) {
		fprintf(stderr, "Need AX.25 port\n");
		return 1;
	}

	if (ax25_config_load_ports() == 0) {
		fprintf(stderr, "No AX.25 ports defined\n");
		return 1;
	}

	if ((portcall = ax25_config_get_addr(port)) == NULL) {
		fprintf(stderr, "Invalid AX.25 port setting - %s\n", port);
		return 1;
	}

	if ((mycall != NULL) && (strcmp(mycall, portcall) != 0)) {
		sprintf(addr, "%s %s", mycall, portcall);
	}
	else {
		strcpy(addr, portcall);
	}
	
	if ((len = ax25_aton(addr, &src)) == -1) {
		fprintf(stderr, "Unable to convert call sign '%s'\n", addr);
		return 1;
	}

	strcpy(addr, dstcall);

	if ((dlen = ax25_aton(addr, &dest)) == -1) {
		fprintf(stderr, "Unable to convert call sign '%s'\n", addr);
		return 1;
	}

	if ((axfd = socket(AF_AX25, SOCK_DGRAM, 0)) < 0) {
		PERROR("dx-broadcast: socket");
		return 1;
	}

	if (bind(axfd, (struct sockaddr *)&src, len) == -1) {
		PERROR("dx-broadcast: bind");
		return 1;
	}

	printf("dx-broadcast: connecting to %s, port %d as %s\n", hp->h_name, ntohs(sin.sin_port), user);
	fflush(stdout);

	len = sizeof(struct sockaddr_in);
	if (connect(ipfd, (struct sockaddr *) &sin, len) == -1) {
		PERROR("dx-broadcast: connect");
		return 1;
	}
	
	if (!daemon_start(FALSE)) {
		fprintf(stderr, "Cannot become a daemon\n");
		return 1;
	}

	while (1) {
		len = read(ipfd, inbuf, sizeof(inbuf) - 1);
		if (len>0)
		{
			inbuf[len] = 0;
			
			if ((login == 0) && (strstr(inbuf, "ogin:") != NULL))
			{
				write(ipfd, user, strlen(user));
				write(ipfd, "\n", 1);
				login = 1;
			}
			else if (strncmp(inbuf, "DX de ", 6) == 0)
			{		
				if (sendto(axfd, inbuf, strlen(inbuf), 0, (struct sockaddr *)&dest, dlen) == -1) {
					fprintf(stderr, "AX.25 socket send failed\n");
					return 1;
				}
			}
		}
	}
}
